/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file vtiinet.h
 *
 * @type helper
 *
 * @description
 *
 * The vtiinet helper provides an interface to the VTi Internet
 * system ("!Voyager").
 *
 * Some of the calls are designed as an interface to the VTiInternet
 * module, whilst the rest are used to create Voyager standard
 * configuration windows.
 *
 * @note It is of little interest to anyone not developing Voyager
 * applications.
 */

#ifndef __rov_vtiinet
#define __rov_vtiinet


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @section VTiInternet SWI interfaces
 * See the VTiInternet documentation for details.
 * @see http://www.argogroup.com/vix/
 */
int VTiInternet_GetErrorCode(void);
canvas VTiInternet_GetPassword(void);
int VTiInternet_CauseError(char * , char * , char * , char * , char *);
int VTiInternet_ReadConfigure(int, int, void * , int);
int VTiInternet_WriteConfigure(int, int, void * , int);
int VTiInternet_EraseConfigure(int, int);
int VTiInternet_EraseProgram(int);
int VTiInternet_CheckFilter(char * , int);
void VTiInternet_SetDriver(char *);
int Driver(int, int, int, int);
void VTi_VSync(void);
#define VTiInternet_ConfigureLength(aa,bb) VTiInternet_ReadConfigure(aa,bb,0,0)
#define VTiInternet_Error1(aa,bb) VTiInternet_CauseError(aa,bb,0,0,0)
#define VTiInternet_Error2(aa,bb,cc) VTiInternet_CauseError(aa,bb,cc,0,0)
#define VTiInternet_Error3(aa,bb,cc,dd) VTiInternet_CauseError(aa,bb,cc,dd,0)
#define VTiInternet_Error4(aa,bb,cc,dd,ee) VTiInternet_CauseError(aa,bb,cc,dd,ee)

extern char VTI_BUTTON[16];
extern char VTI_PASSWORD[16];
extern char VTI_PASSTYPE[20];
extern char VTI_PASSTEXT[256];

/**
 * Convert a four character ID into a word.
 * @mparam a the bottom 8 bits.
 * @mparam b the next 8 bits.
 * @mparam c the next 8 bits.
 * @mparam d the top 8 bits.
 * @note This is used to make IDs for the configuration file. For example, to
 * make the id 'NEWS', use ID('N', 'E', 'W', 'S').
 */
#define ID(a, b, c, d) ((int) ((int) d << 24) | ((int) c << 16) | ((int) b <<  8) | (int)(a))


/**
 * Definition of a menu maker
 * @param ic icon to which the menu is attached.
 * @return the generated menu.
 * @note This is called when an icon generated by VTi_CreateMenu() is clicked
 * on.
 * @see VTi_CreateMenu()
 */
typedef menu (*menu_make)(icon ic);

/**
 * Definition of an option handler.
 * @note These are called to read, write or cancel options
 */
typedef void (* _opts)(void);

/**
 * Definition of an extendable list handler.
 * @param event the event code.
 * @return non-zero for event handled, zero for event not handled.
 */
typedef int (* list_addop)(int event);

/**
 * Create a configuration window.
 * @param title the title of the window.
 * @param h the height of the nested window.
 * @param wr the function called to write current options.
 * @param rd the function called to read options.
 * @param cn the function called to cancel options.
 * @param vixen the Vixen ID for this configuration program.
 * @see http://www.argonet.co.uk/vixen/
 * @note The VTiInet system maintains the current y coordinate for
 * all the icons on the configuration screen.
 */
void VTi_OpenConfigure(char *title,
		       int h,
		       _opts wr,
		       _opts rd,
		       _opts cn,
		       int vixen);

/**
 * Create a menu item in the configuration window.
 * @param text the short descriptive text for the menu item.
 * @param value the pointer to the buffer holding the selected value.
 * @param maker the menu maker.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This creates a half-width menu icon, with the descriptive
 * text to the left of the value box.
 */
icon VTi_CreateMenu(char *text,
		    char *value,
		    menu_make maker,
		    char *help);

/**
 * Create a menu item in the configuration window.
 * @param text the short descriptive text for the menu item.
 * @param value the pointer to the buffer holding the selected value.
 * @param maker the menu maker.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This creates a full-width menu icon, with the descriptive
 * text above the value box.
 */
icon VTi_CreateLongMenu(char *text,
			char *value,
			menu_make maker,
			char *help);

/**
 * Fade a menu item
 * @param ic the icon to fade.
 * @param faded non-zero for faded, zero for unfaded.
 * @note This also fades the value box as well.
 */
void VTi_FadeMenu(icon ic,
		  int faded);

/**
 * Create a writeable icon.
 * @param text the short descriptive text for the icon.
 * @param value the pointer to the value buffer.
 * @param len the maximum length of the string.
 * @param type the type of the writeable field.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This creates a half-width writeable icon, with the descriptive
 * text to the left of the value box.
 * @see Writeable icon types
 * @see VTi_CreateLongWriteable()
 * @see VTi_CreateUnits() 
*/
icon VTi_CreateWriteable(char *text,
			 char *value,
			 int len,
			 int type,
			 char *help);

/**
 * Create a writeable icon.
 * @param text the short descriptive text for the icon.
 * @param value the pointer to the value buffer.
 * @param len the maximum length of the string.
 * @param type the type of the writeable field.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This creates a full-width writeable icon, with the descriptive
 * text above the value box.
 * @see Writeable icon types
 * @see VTi_CreateWriteable()
 * @see VTi_CreateUnits()
 */
icon VTi_CreateLongWriteable(char *text,
			     char *value,
			     int len,
			     int type,
			     char *help);
/**
 * Create a horizontal bar.
 */
void VTi_CreateHBar(void);

/**
 * Create a comment.
 * @param text the text for the comment.
 */
void VTi_CreateComment(char *text);

/**
 * Create a writeable icon with units at the end.
 * @param text the short descriptive text for the icon.
 * @param value the pointer to the value buffer.
 * @param units the text for the units.
 * @param len the maximum length of the string.
 * @param type the type of the writeable field.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This creates a full-width writeable icon, with the descriptive
 * text above the value box.
 * @see Writeable icon types
 * @see VTi_CreateWriteable()
 * @see VTi_CreateLongWriteable()
 */
icon VTi_CreateUnits(char *text,
		     char *value,
		     char *units,
		     int len,
		     int type,
		     char *help);
/**
 * Create an option box
 * @param text the text to the right of the option box
 * @param esg the ESG for the option box.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note If the ESG is zero, then an option icon is created. Otherwise, a radio
 * icon is created.
 * @see VTi_CreateShortOption()
 */
icon VTi_CreateOption(char *text,
		      int esg,
		      char *help);

/**
 * Create a short option box
 * @param text the text to the right of the option box
 * @param esg the ESG for the option box.
 * @param help the help text returned to interactive help.
 * @return the generated icon.
 * @note This is designed to allow two option boxes across the
 * configuration window.
 * @note If the ESG is zero, then an option icon is created. Otherwise, a radio
 * icon is created.
 * @see VTi_CreateOption()
 * @see VTi_ReleaseShort()
 */
icon VTi_CreateShortOption(char *text,
			   int esg,
			   char *help);

/**
 * Release the short flag.
 * @note This clears an internal variable, and is used if an
 * odd number of short options have been created.
 * @see VTi_CreateShortOption()
 */
void VTi_ReleaseShort(void);

/**
 * Create an information field.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the information value buffer.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a half-width information icon, with the descriptive
 * text to the left of the value box.
 */
icon VTi_CreateInformation(char *text,
			   char *value,
			   char *help);

/**
 * Create a long information field.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the information value buffer.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a full-width information icon, with the descriptive
 * text above the value box.
 */
icon VTi_CreateLargeInformation(char *text,
				char *value,
				char *help);

/**
 * Return interactive help
 * @note This is called by the canvas' help event handler.
 */
void VTi_Help(void);

/**
 * Create an extendible list.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the value buffer.
 * @param maker the menu maker.
 * @param adder the list adder handler.
 * @param len the length of the text in the value buffer.
 * @param type the type of the writeable icon.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a half-width information icon, with the descriptive
 * text to the left of the value box.
 * @see VTi_CreateOpenExtendibleList()
 * @see VTi_CreateLargeExtendibleList()
 * @see menu_make()
 * @see list_addop()
 * @see Writeable icon types
 */
icon VTi_CreateExtendibleList(char *text,
			      char *value,
			      menu_make maker,
			      list_addop adder,
			      int len,
			      int type,
			      char * help);

/**
 * Create a large extendible list.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the value buffer.
 * @param maker the menu maker.
 * @param adder the list adder handler.
 * @param len the length of the text in the value buffer.
 * @param type the type of the writeable icon.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a full-width information icon, with the descriptive
 * text above the value box.
 * @see VTi_CreateExtendibleList()
 * @see VTi_CreateOpenLargeExtendibleList()
 * @see menu_make()
 * @see list_addop()
 * @see Writeable icon types
 */
icon VTi_CreateLargeExtendibleList(char *text,
				   char *value,
				   menu_make maker,
				   list_addop adder,
				   int len,
				   int type,
				   char * help);

/**
 * Fade an extendible list
 * @param ic the icon.
 * @param faded non-zero for faded, zero for unfaded.
 */
void VTi_FadeExtendibleList(icon ic,
			    int faded);

/**
 * Create an action button.
 * @param text the text in the button.
 * @param help the pointer to the help text.
 * @return the generated icon.
 */
icon VTi_CreateAction(char *text,
		      char *help);

/**
 * Create an open extendible list.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the value buffer.
 * @param maker the menu maker.
 * @param adder the list adder handler.
 * @param len the length of the text in the value buffer.
 * @param type the type of the writeable icon.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a half-width information icon, with the descriptive
 * text to the left of the value box.
 * @see VTi_CreateLargeExtendibleList()
 * @see VTi_CreateOpenLargeExtendibleList()
 * @see VTi_AddExtendibleList()
 * @see VTi_CloseExtendibleList()
 * @see menu_make()
 * @see list_addop()
 * @see Writeable icon types
 * @note This differs from a normal extendible list in that there can be
 * other icons between the extendible list and the "add" and "remove"
 * buttons.
 */
icon VTi_CreateOpenExtendibleList(char *text,
				  char *value,
				  menu_make maker,
				  list_addop adder,
				  int len,
				  int type,
				  char * help);

/**
 * Create a large extendible list.
 * @param text the short descriptive text for the field.
 * @param value the pointer to the value buffer.
 * @param maker the menu maker.
 * @param adder the list adder handler.
 * @param len the length of the text in the value buffer.
 * @param type the type of the writeable icon.
 * @param help the pointer to the help text.
 * @return the generated icon.
 * @note This creates a full-width information icon, with the descriptive
 * text above the value box.
 * @see VTi_CreateOpenExtendibleList()
 * @see VTi_CreateLargeExtendibleList()
 * @see VTi_AddExtendibleList()
 * @see VTi_CloseExtendibleList()
 * @see menu_make()
 * @see list_addop()
 * @see Writeable icon types
 * @note This differs from a normal extendible list in that there can be
 * other icons between the extendible list and the "add" and "remove"
 * buttons.
 */
icon VTi_CreateOpenLargeExtendibleList(char *text,
				       char *value,
				       menu_make maker,
				       list_addop adder,
				       int len,
				       int type,
				       char *help);

/**
 * Add to an extendible list.
 * @param text the short descriptive text.
 * @param value the pointer to the value buffer.
 * @param len the length of the buffer.
 * @param type the type if the writeable icon.
 * @return the generated icon.
 * @see VTi_CreateOpenExtendibleList()
 * @see VTi_CreateOpenLargeExtendibleList()
 */
icon VTi_AddToExtendibleList(char *text,
			     char *value,
			     int len,
			     int type);

/**
 * Terminate an extendible list by adding "Add" and "Remove" icons.
 */
void VTi_CloseExtendibleList(void);

/**
 * Create a large information field in an open extendible list.
 * @param text the short descriptive text.
 * @param value the pointer to the value buffer.
 */
icon VTi_CreateOpenLargeInformation(char *text,
				    char * value);

/**
 * Create an example text.
 * @param text the short descriptive text.
 * @param help the pointer to the help text.
 */
icon VTi_CreateExample(char *text,
		       char *help);

/**
 * Get/change the y position.
 * @param offset the offset to change the size by.
 * @return the next coordinate for an icon.
 */
int VTi_Position(int offset);

/**
 * @section Writeable icon types
 */
#define VTI_ALL    (0)
#define VTI_DIGITS (1)
#define VTI_ALPHA  (2)
#define VTI_ALNUM  (3)
#define VTI_HEX    (4)
#define VTI_GROUP  (5)
#define VTI_IP	   (6)

#define LISTOP_UP     (0)
#define LISTOP_DOWN   (1)
#define LISTOP_ADD    (2)
#define LISTOP_REMOVE (3)

#define LIST_NOFADE (0)
#define LIST_PARTIALFADE (1)
#define LIST_FULLFADE (2)

#define FILTER_WEB (0)
#define FILTER_IP (1)
#define FILTER_MAIL (2)
#define FILTER_NEWS (3)


#ifdef __cplusplus
}
#endif

#endif
